# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ServiceType(str, enum.Enum):
    NEW_PATIENT_VIDEO_APPT = "new_patient_video_appt"
    NEW_PATIENT_PREV_VIDEO_APPT = "new_patient_prev_video_appt"
    NEW_PATIENT_OFFICE_APPT = "new_patient_office_appt"
    ESTABLISHED_PATIENT_VIDEO_APPT = "established_patient_video_appt"
    ESTABLISHED_PATIENT_PREV_VIDEO_APPT = "established_patient_prev_video_appt"
    ESTABLISHED_PATIENT_OFFICE_APPT = "established_patient_office_appt"
    PATIENT_PORTAL_WITH_QUALIFIED_PHYSICIAN = "patient_portal_with_qualified_physician"
    PATIENT_PORTAL_WITH_NON_QUALIFIED_PHYSICIAN = "patient_portal_with_non_qualified_physician"
    GROUP_THERAPY = "group_therapy"
    PHONE_APPT = "phone_appt"
    ANNUAL_WELLNESS_VISIT = "annual_wellness_visit"
    REMOTE_PATIENT_MONITORING = "remote_patient_monitoring"
    DIAGNOSTIC = "diagnostic"
    CARDIAC_MONITORING = "cardiac_monitoring"
    CARDIAC_CODES = "cardiac_codes"
    PSYCHOTHERAPY = "psychotherapy"
    PHONE_THERAPY = "phone_therapy"
    ALCOHOL_ABUSE = "alcohol_abuse"
    SMOKING_CESSATION = "smoking_cessation"
    SPEECH_THERAPY = "speech_therapy"
    HOME_VISIT = "home_visit"
    MEDICAL_NUTRITION_THERAPY = "medical_nutrition_therapy"
    NUTRITIONAL_COUNSELING = "nutritional_counseling"
    DIETITIAN_VISIT = "dietitian_visit"
    PREVENTATIVE_MEDICINE_COUNSELING = "preventative_medicine_counseling"
    CHRONIC_CARE = "chronic_care"
    ADVANCE_CARE = "advance_care"
    COLLABORATIVE_CARE_MANAGEMENT = "collaborative_care_management"
    ABORTION_CARE = "abortion_care"
    ABORTION_DRUGS = "abortion_drugs"
    REMOTE_THERAPEUTIC_MONITORING = "remote_therapeutic_monitoring"
    PRINCIPAL_CARE_MGMT = "principal_care_mgmt"
    OTHER_EM = "other_em"
    TELEHEALTH_OUD = "telehealth_oud"
    SUBSTANCE_ABUSE_TREATMENT = "substance_abuse_treatment"
    ACUPUNCTURE = "acupuncture"
    PHYSICAL_THERAPY = "physical_therapy"
    LACTATION_CONSULTING = "lactation_consulting"
    SLEEP_STUDY = "sleep_study"
    CMS_TELEHEALTH_CODES = "cms_telehealth_codes"
    CMS_TELEHEALTH_AUDIO_CODES = "cms_telehealth_audio_codes"

    def visit(
        self,
        new_patient_video_appt: typing.Callable[[], T_Result],
        new_patient_prev_video_appt: typing.Callable[[], T_Result],
        new_patient_office_appt: typing.Callable[[], T_Result],
        established_patient_video_appt: typing.Callable[[], T_Result],
        established_patient_prev_video_appt: typing.Callable[[], T_Result],
        established_patient_office_appt: typing.Callable[[], T_Result],
        patient_portal_with_qualified_physician: typing.Callable[[], T_Result],
        patient_portal_with_non_qualified_physician: typing.Callable[[], T_Result],
        group_therapy: typing.Callable[[], T_Result],
        phone_appt: typing.Callable[[], T_Result],
        annual_wellness_visit: typing.Callable[[], T_Result],
        remote_patient_monitoring: typing.Callable[[], T_Result],
        diagnostic: typing.Callable[[], T_Result],
        cardiac_monitoring: typing.Callable[[], T_Result],
        cardiac_codes: typing.Callable[[], T_Result],
        psychotherapy: typing.Callable[[], T_Result],
        phone_therapy: typing.Callable[[], T_Result],
        alcohol_abuse: typing.Callable[[], T_Result],
        smoking_cessation: typing.Callable[[], T_Result],
        speech_therapy: typing.Callable[[], T_Result],
        home_visit: typing.Callable[[], T_Result],
        medical_nutrition_therapy: typing.Callable[[], T_Result],
        nutritional_counseling: typing.Callable[[], T_Result],
        dietitian_visit: typing.Callable[[], T_Result],
        preventative_medicine_counseling: typing.Callable[[], T_Result],
        chronic_care: typing.Callable[[], T_Result],
        advance_care: typing.Callable[[], T_Result],
        collaborative_care_management: typing.Callable[[], T_Result],
        abortion_care: typing.Callable[[], T_Result],
        abortion_drugs: typing.Callable[[], T_Result],
        remote_therapeutic_monitoring: typing.Callable[[], T_Result],
        principal_care_mgmt: typing.Callable[[], T_Result],
        other_em: typing.Callable[[], T_Result],
        telehealth_oud: typing.Callable[[], T_Result],
        substance_abuse_treatment: typing.Callable[[], T_Result],
        acupuncture: typing.Callable[[], T_Result],
        physical_therapy: typing.Callable[[], T_Result],
        lactation_consulting: typing.Callable[[], T_Result],
        sleep_study: typing.Callable[[], T_Result],
        cms_telehealth_codes: typing.Callable[[], T_Result],
        cms_telehealth_audio_codes: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ServiceType.NEW_PATIENT_VIDEO_APPT:
            return new_patient_video_appt()
        if self is ServiceType.NEW_PATIENT_PREV_VIDEO_APPT:
            return new_patient_prev_video_appt()
        if self is ServiceType.NEW_PATIENT_OFFICE_APPT:
            return new_patient_office_appt()
        if self is ServiceType.ESTABLISHED_PATIENT_VIDEO_APPT:
            return established_patient_video_appt()
        if self is ServiceType.ESTABLISHED_PATIENT_PREV_VIDEO_APPT:
            return established_patient_prev_video_appt()
        if self is ServiceType.ESTABLISHED_PATIENT_OFFICE_APPT:
            return established_patient_office_appt()
        if self is ServiceType.PATIENT_PORTAL_WITH_QUALIFIED_PHYSICIAN:
            return patient_portal_with_qualified_physician()
        if self is ServiceType.PATIENT_PORTAL_WITH_NON_QUALIFIED_PHYSICIAN:
            return patient_portal_with_non_qualified_physician()
        if self is ServiceType.GROUP_THERAPY:
            return group_therapy()
        if self is ServiceType.PHONE_APPT:
            return phone_appt()
        if self is ServiceType.ANNUAL_WELLNESS_VISIT:
            return annual_wellness_visit()
        if self is ServiceType.REMOTE_PATIENT_MONITORING:
            return remote_patient_monitoring()
        if self is ServiceType.DIAGNOSTIC:
            return diagnostic()
        if self is ServiceType.CARDIAC_MONITORING:
            return cardiac_monitoring()
        if self is ServiceType.CARDIAC_CODES:
            return cardiac_codes()
        if self is ServiceType.PSYCHOTHERAPY:
            return psychotherapy()
        if self is ServiceType.PHONE_THERAPY:
            return phone_therapy()
        if self is ServiceType.ALCOHOL_ABUSE:
            return alcohol_abuse()
        if self is ServiceType.SMOKING_CESSATION:
            return smoking_cessation()
        if self is ServiceType.SPEECH_THERAPY:
            return speech_therapy()
        if self is ServiceType.HOME_VISIT:
            return home_visit()
        if self is ServiceType.MEDICAL_NUTRITION_THERAPY:
            return medical_nutrition_therapy()
        if self is ServiceType.NUTRITIONAL_COUNSELING:
            return nutritional_counseling()
        if self is ServiceType.DIETITIAN_VISIT:
            return dietitian_visit()
        if self is ServiceType.PREVENTATIVE_MEDICINE_COUNSELING:
            return preventative_medicine_counseling()
        if self is ServiceType.CHRONIC_CARE:
            return chronic_care()
        if self is ServiceType.ADVANCE_CARE:
            return advance_care()
        if self is ServiceType.COLLABORATIVE_CARE_MANAGEMENT:
            return collaborative_care_management()
        if self is ServiceType.ABORTION_CARE:
            return abortion_care()
        if self is ServiceType.ABORTION_DRUGS:
            return abortion_drugs()
        if self is ServiceType.REMOTE_THERAPEUTIC_MONITORING:
            return remote_therapeutic_monitoring()
        if self is ServiceType.PRINCIPAL_CARE_MGMT:
            return principal_care_mgmt()
        if self is ServiceType.OTHER_EM:
            return other_em()
        if self is ServiceType.TELEHEALTH_OUD:
            return telehealth_oud()
        if self is ServiceType.SUBSTANCE_ABUSE_TREATMENT:
            return substance_abuse_treatment()
        if self is ServiceType.ACUPUNCTURE:
            return acupuncture()
        if self is ServiceType.PHYSICAL_THERAPY:
            return physical_therapy()
        if self is ServiceType.LACTATION_CONSULTING:
            return lactation_consulting()
        if self is ServiceType.SLEEP_STUDY:
            return sleep_study()
        if self is ServiceType.CMS_TELEHEALTH_CODES:
            return cms_telehealth_codes()
        if self is ServiceType.CMS_TELEHEALTH_AUDIO_CODES:
            return cms_telehealth_audio_codes()



# This file was auto-generated by Fern from our API Definition.

from . import common, coverages, patients
from .common import (
    Address,
    AddressUse,
    ContactPoint,
    ContactPointUse,
    DisabilityStatus,
    ErrorBase,
    Ethnicity,
    ExternalProvider,
    ExternalProviderType,
    Gender,
    HumanName,
    NameUse,
    NotFoundError,
    NotFoundErrorBody,
    OrganizationId,
    PayerId,
    Period,
    Race,
    Relationship,
    Sex,
    SexualOrientation,
    UserId,
    VersionConflictError,
    VersionConflictErrorBody,
)

__all__ = [
    "Address",
    "AddressUse",
    "ContactPoint",
    "ContactPointUse",
    "DisabilityStatus",
    "ErrorBase",
    "Ethnicity",
    "ExternalProvider",
    "ExternalProviderType",
    "Gender",
    "HumanName",
    "NameUse",
    "NotFoundError",
    "NotFoundErrorBody",
    "OrganizationId",
    "PayerId",
    "Period",
    "Race",
    "Relationship",
    "Sex",
    "SexualOrientation",
    "UserId",
    "VersionConflictError",
    "VersionConflictErrorBody",
    "common",
    "coverages",
    "patients",
]



# This file was auto-generated by Fern from our API Definition.

from .coverage import Coverage
from .coverage_id import CoverageId
from .coverage_status import CoverageStatus
from .insurance_plan import InsurancePlan
from .insurance_type_code import InsuranceTypeCode
from .mutable_coverage import MutableCoverage
from .network_type import NetworkType
from .subscriber import Subscriber

__all__ = [
    "Coverage",
    "CoverageId",
    "CoverageStatus",
    "InsurancePlan",
    "InsuranceTypeCode",
    "MutableCoverage",
    "NetworkType",
    "Subscriber",
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class InvoiceItemUpdateType(str, enum.Enum):
    APPEND = "APPEND"
    OVERWRITE = "OVERWRITE"

    def visit(self, append: typing.Callable[[], T_Result], overwrite: typing.Callable[[], T_Result]) -> T_Result:
        if self is InvoiceItemUpdateType.APPEND:
            return append()
        if self is InvoiceItemUpdateType.OVERWRITE:
            return overwrite()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class Ethnicity(str, enum.Enum):
    HISPANIC_OR_LATINO = "HISPANIC_OR_LATINO"
    NOT_HISPANIC_OR_LATINO = "NOT_HISPANIC_OR_LATINO"
    UNKNOWN = "UNKNOWN"
    REFUSED = "REFUSED"

    def visit(
        self,
        hispanic_or_latino: typing.Callable[[], T_Result],
        not_hispanic_or_latino: typing.Callable[[], T_Result],
        unknown: typing.Callable[[], T_Result],
        refused: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is Ethnicity.HISPANIC_OR_LATINO:
            return hispanic_or_latino()
        if self is Ethnicity.NOT_HISPANIC_OR_LATINO:
            return not_hispanic_or_latino()
        if self is Ethnicity.UNKNOWN:
            return unknown()
        if self is Ethnicity.REFUSED:
            return refused()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class NoteCategory(str, enum.Enum):
    CLINICAL = "clinical"
    CARE_PLAN = "care_plan"
    DIAGNOSES = "diagnoses"
    VITALS = "vitals"
    PHYSICAL_EXAM = "physical_exam"
    REVIEW_OF_SYSTEMS = "review_of_systems"
    MEDICAL_DECISIONS = "medical_decisions"
    HISTORY_OF_PRESENT_ILLNESS = "history_of_present_illness"
    PATIENT_INFO = "patient_info"
    CHIEF_COMPLAINT = "chief_complaint"
    HEALTH_RECORD = "health_record"
    CONSENT = "consent"
    PROCEDURE = "procedure"
    TIME_IN_APPOINTMENT = "time_in_appointment"

    def visit(
        self,
        clinical: typing.Callable[[], T_Result],
        care_plan: typing.Callable[[], T_Result],
        diagnoses: typing.Callable[[], T_Result],
        vitals: typing.Callable[[], T_Result],
        physical_exam: typing.Callable[[], T_Result],
        review_of_systems: typing.Callable[[], T_Result],
        medical_decisions: typing.Callable[[], T_Result],
        history_of_present_illness: typing.Callable[[], T_Result],
        patient_info: typing.Callable[[], T_Result],
        chief_complaint: typing.Callable[[], T_Result],
        health_record: typing.Callable[[], T_Result],
        consent: typing.Callable[[], T_Result],
        procedure: typing.Callable[[], T_Result],
        time_in_appointment: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is NoteCategory.CLINICAL:
            return clinical()
        if self is NoteCategory.CARE_PLAN:
            return care_plan()
        if self is NoteCategory.DIAGNOSES:
            return diagnoses()
        if self is NoteCategory.VITALS:
            return vitals()
        if self is NoteCategory.PHYSICAL_EXAM:
            return physical_exam()
        if self is NoteCategory.REVIEW_OF_SYSTEMS:
            return review_of_systems()
        if self is NoteCategory.MEDICAL_DECISIONS:
            return medical_decisions()
        if self is NoteCategory.HISTORY_OF_PRESENT_ILLNESS:
            return history_of_present_illness()
        if self is NoteCategory.PATIENT_INFO:
            return patient_info()
        if self is NoteCategory.CHIEF_COMPLAINT:
            return chief_complaint()
        if self is NoteCategory.HEALTH_RECORD:
            return health_record()
        if self is NoteCategory.CONSENT:
            return consent()
        if self is NoteCategory.PROCEDURE:
            return procedure()
        if self is NoteCategory.TIME_IN_APPOINTMENT:
            return time_in_appointment()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.jsonable_encoder import jsonable_encoder
from .....core.pydantic_utilities import pydantic_v1
from .....core.request_options import RequestOptions
from ....commons.errors.http_request_validations_error import HttpRequestValidationsError
from ....commons.types.email import Email
from ....commons.types.encounter_id import EncounterId
from ....commons.types.phone_number import PhoneNumber
from ....commons.types.request_validation_error import RequestValidationError
from ....commons.types.street_address_short_zip import StreetAddressShortZip
from .errors.encounter_has_existing_guarantor_error import EncounterHasExistingGuarantorError
from .types.encounter_has_existing_guarantor_error_type import EncounterHasExistingGuarantorErrorType
from .types.guarantor import Guarantor
from .types.guarantor_create import GuarantorCreate
from .types.guarantor_id import GuarantorId

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        encounter_id: EncounterId,
        *,
        request: GuarantorCreate,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Guarantor:
        """
        Creates a new guarantor and returns the newly created Guarantor object.

        Parameters
        ----------
        encounter_id : EncounterId

        request : GuarantorCreate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Guarantor

        Examples
        --------
        import datetime
        import uuid

        from candid import PhoneNumber, PhoneNumberType, State, StreetAddressShortZip
        from candid.client import CandidApiClient
        from candid.resources.guarantor.v_1 import GuarantorCreate

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.guarantor.v_1.create(
            encounter_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            request=GuarantorCreate(
                phone_numbers=[
                    PhoneNumber(
                        number="1234567890",
                        type=PhoneNumberType.HOME,
                    )
                ],
                phone_consent=True,
                email="johndoe@joincandidhealth.com",
                email_consent=True,
                first_name="string",
                last_name="string",
                external_id="string",
                date_of_birth=datetime.date.fromisoformat(
                    "2023-01-15",
                ),
                address=StreetAddressShortZip(
                    address_1="123 Main St",
                    address_2="Apt 1",
                    city="New York",
                    state=State.NY,
                    zip_code="10001",
                    zip_plus_four_code="1234",
                ),
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/guarantors/v1/{jsonable_encoder(encounter_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Guarantor, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EncounterHasExistingGuarantorError":
                raise EncounterHasExistingGuarantorError(
                    pydantic_v1.parse_obj_as(EncounterHasExistingGuarantorErrorType, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "HttpRequestValidationsError":
                raise HttpRequestValidationsError(
                    pydantic_v1.parse_obj_as(typing.List[RequestValidationError], _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, guarantor_id: GuarantorId, *, request_options: typing.Optional[RequestOptions] = None) -> Guarantor:
        """
        Retrieves a guarantor by its `guarantor_id`.

        Parameters
        ----------
        guarantor_id : GuarantorId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Guarantor

        Examples
        --------
        import uuid

        from candid.client import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.guarantor.v_1.get(
            guarantor_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/guarantors/v1/{jsonable_encoder(guarantor_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Guarantor, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        guarantor_id: GuarantorId,
        *,
        first_name: typing.Optional[str] = OMIT,
        last_name: typing.Optional[str] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        date_of_birth: typing.Optional[dt.date] = OMIT,
        address: typing.Optional[StreetAddressShortZip] = OMIT,
        phone_numbers: typing.Optional[typing.Sequence[PhoneNumber]] = OMIT,
        phone_consent: typing.Optional[bool] = OMIT,
        email: typing.Optional[Email] = OMIT,
        email_consent: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Guarantor:
        """
        Updates a guarantor by its `guarantor_id`.

        Parameters
        ----------
        guarantor_id : GuarantorId

        first_name : typing.Optional[str]

        last_name : typing.Optional[str]

        external_id : typing.Optional[str]
            A unique identifier for the guarantor assigned by an external system.

        date_of_birth : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-25.


        address : typing.Optional[StreetAddressShortZip]

        phone_numbers : typing.Optional[typing.Sequence[PhoneNumber]]

        phone_consent : typing.Optional[bool]

        email : typing.Optional[Email]

        email_consent : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Guarantor

        Examples
        --------
        import datetime
        import uuid

        from candid import PhoneNumber, PhoneNumberType, State, StreetAddressShortZip
        from candid.client import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.guarantor.v_1.update(
            guarantor_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            first_name="string",
            last_name="string",
            external_id="string",
            date_of_birth=datetime.date.fromisoformat(
                "2023-01-15",
            ),
            address=StreetAddressShortZip(
                address_1="123 Main St",
                address_2="Apt 1",
                city="New York",
                state=State.NY,
                zip_code="10001",
                zip_plus_four_code="1234",
            ),
            phone_numbers=[
                PhoneNumber(
                    number="1234567890",
                    type=PhoneNumberType.HOME,
                )
            ],
            phone_consent=True,
            email="johndoe@joincandidhealth.com",
            email_consent=True,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/guarantors/v1/{jsonable_encoder(guarantor_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json={
                "first_name": first_name,
                "last_name": last_name,
                "external_id": external_id,
                "date_of_birth": date_of_birth,
                "address": address,
                "phone_numbers": phone_numbers,
                "phone_consent": phone_consent,
                "email": email,
                "email_consent": email_consent,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Guarantor, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        encounter_id: EncounterId,
        *,
        request: GuarantorCreate,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Guarantor:
        """
        Creates a new guarantor and returns the newly created Guarantor object.

        Parameters
        ----------
        encounter_id : EncounterId

        request : GuarantorCreate

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Guarantor

        Examples
        --------
        import asyncio
        import datetime
        import uuid

        from candid import PhoneNumber, PhoneNumberType, State, StreetAddressShortZip
        from candid.client import AsyncCandidApiClient
        from candid.resources.guarantor.v_1 import GuarantorCreate

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.guarantor.v_1.create(
                encounter_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                request=GuarantorCreate(
                    phone_numbers=[
                        PhoneNumber(
                            number="1234567890",
                            type=PhoneNumberType.HOME,
                        )
                    ],
                    phone_consent=True,
                    email="johndoe@joincandidhealth.com",
                    email_consent=True,
                    first_name="string",
                    last_name="string",
                    external_id="string",
                    date_of_birth=datetime.date.fromisoformat(
                        "2023-01-15",
                    ),
                    address=StreetAddressShortZip(
                        address_1="123 Main St",
                        address_2="Apt 1",
                        city="New York",
                        state=State.NY,
                        zip_code="10001",
                        zip_plus_four_code="1234",
                    ),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/guarantors/v1/{jsonable_encoder(encounter_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Guarantor, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EncounterHasExistingGuarantorError":
                raise EncounterHasExistingGuarantorError(
                    pydantic_v1.parse_obj_as(EncounterHasExistingGuarantorErrorType, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "HttpRequestValidationsError":
                raise HttpRequestValidationsError(
                    pydantic_v1.parse_obj_as(typing.List[RequestValidationError], _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, guarantor_id: GuarantorId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> Guarantor:
        """
        Retrieves a guarantor by its `guarantor_id`.

        Parameters
        ----------
        guarantor_id : GuarantorId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Guarantor

        Examples
        --------
        import asyncio
        import uuid

        from candid.client import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.guarantor.v_1.get(
                guarantor_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/guarantors/v1/{jsonable_encoder(guarantor_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Guarantor, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        guarantor_id: GuarantorId,
        *,
        first_name: typing.Optional[str] = OMIT,
        last_name: typing.Optional[str] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        date_of_birth: typing.Optional[dt.date] = OMIT,
        address: typing.Optional[StreetAddressShortZip] = OMIT,
        phone_numbers: typing.Optional[typing.Sequence[PhoneNumber]] = OMIT,
        phone_consent: typing.Optional[bool] = OMIT,
        email: typing.Optional[Email] = OMIT,
        email_consent: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Guarantor:
        """
        Updates a guarantor by its `guarantor_id`.

        Parameters
        ----------
        guarantor_id : GuarantorId

        first_name : typing.Optional[str]

        last_name : typing.Optional[str]

        external_id : typing.Optional[str]
            A unique identifier for the guarantor assigned by an external system.

        date_of_birth : typing.Optional[dt.date]
            Date formatted as YYYY-MM-DD; eg: 2019-08-25.


        address : typing.Optional[StreetAddressShortZip]

        phone_numbers : typing.Optional[typing.Sequence[PhoneNumber]]

        phone_consent : typing.Optional[bool]

        email : typing.Optional[Email]

        email_consent : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Guarantor

        Examples
        --------
        import asyncio
        import datetime
        import uuid

        from candid import PhoneNumber, PhoneNumberType, State, StreetAddressShortZip
        from candid.client import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.guarantor.v_1.update(
                guarantor_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                first_name="string",
                last_name="string",
                external_id="string",
                date_of_birth=datetime.date.fromisoformat(
                    "2023-01-15",
                ),
                address=StreetAddressShortZip(
                    address_1="123 Main St",
                    address_2="Apt 1",
                    city="New York",
                    state=State.NY,
                    zip_code="10001",
                    zip_plus_four_code="1234",
                ),
                phone_numbers=[
                    PhoneNumber(
                        number="1234567890",
                        type=PhoneNumberType.HOME,
                    )
                ],
                phone_consent=True,
                email="johndoe@joincandidhealth.com",
                email_consent=True,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/guarantors/v1/{jsonable_encoder(guarantor_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json={
                "first_name": first_name,
                "last_name": last_name,
                "external_id": external_id,
                "date_of_birth": date_of_birth,
                "address": address,
                "phone_numbers": phone_numbers,
                "phone_consent": phone_consent,
                "email": email,
                "email_consent": email_consent,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(Guarantor, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ......core.datetime_utils import serialize_datetime
from ......core.pydantic_utilities import deep_union_pydantic_dicts
from .....commons.types.rate_id import RateId
from .....commons.types.user_id import UserId
from .dimensions import Dimensions
from .rate_entry import RateEntry


class Rate(pydantic.BaseModel):
    """
    A comprehensive rate including the current rate value and all values for historic time ranges. The time ranges specified by each RateEntry are disjoint. A rate must always have at least one entry.
    """

    rate_id: RateId
    dimensions: Dimensions = pydantic.Field()
    """
    The dimension values that distinguish this rate from others.
    """

    version: int = pydantic.Field()
    """
    The version of this rate in the system.
    """

    updated_at: dt.date
    updated_by: UserId
    entries: typing.List[RateEntry]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.forbid
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class PatientPaymentStatus(str, enum.Enum):
    PENDING = "PENDING"
    PAID = "paid"
    CANCELED = "CANCELED"
    VOIDED = "voided"
    FAILED = "FAILED"
    COMPLETED = "COMPLETED"
    STRIPE_SUCCEEDED = "succeeded"
    STRIPE_PENDING = "pending"
    STRIPE_FAILED = "failed"
    STRIPE_REQUIRES_ACTION = "requires_action"
    STRIPE_CANCELED = "canceled"

    def visit(
        self,
        pending: typing.Callable[[], T_Result],
        paid: typing.Callable[[], T_Result],
        canceled: typing.Callable[[], T_Result],
        voided: typing.Callable[[], T_Result],
        failed: typing.Callable[[], T_Result],
        completed: typing.Callable[[], T_Result],
        stripe_succeeded: typing.Callable[[], T_Result],
        stripe_pending: typing.Callable[[], T_Result],
        stripe_failed: typing.Callable[[], T_Result],
        stripe_requires_action: typing.Callable[[], T_Result],
        stripe_canceled: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is PatientPaymentStatus.PENDING:
            return pending()
        if self is PatientPaymentStatus.PAID:
            return paid()
        if self is PatientPaymentStatus.CANCELED:
            return canceled()
        if self is PatientPaymentStatus.VOIDED:
            return voided()
        if self is PatientPaymentStatus.FAILED:
            return failed()
        if self is PatientPaymentStatus.COMPLETED:
            return completed()
        if self is PatientPaymentStatus.STRIPE_SUCCEEDED:
            return stripe_succeeded()
        if self is PatientPaymentStatus.STRIPE_PENDING:
            return stripe_pending()
        if self is PatientPaymentStatus.STRIPE_FAILED:
            return stripe_failed()
        if self is PatientPaymentStatus.STRIPE_REQUIRES_ACTION:
            return stripe_requires_action()
        if self is PatientPaymentStatus.STRIPE_CANCELED:
            return stripe_canceled()



# This file was auto-generated by Fern from our API Definition.

from .types import (
    Gender,
    IndividualBase,
    IndividualId,
    Patient,
    PatientBase,
    PatientCreate,
    Subscriber,
    SubscriberBase,
    SubscriberCreate,
)

__all__ = [
    "Gender",
    "IndividualBase",
    "IndividualId",
    "Patient",
    "PatientBase",
    "PatientCreate",
    "Subscriber",
    "SubscriberBase",
    "SubscriberCreate",
]



# This file was auto-generated by Fern from our API Definition.

from .encounter_service_facility import EncounterServiceFacility
from .encounter_service_facility_base import EncounterServiceFacilityBase
from .service_facility_id import ServiceFacilityId

__all__ = ["EncounterServiceFacility", "EncounterServiceFacilityBase", "ServiceFacilityId"]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import datetime as dt
import typing

import pydantic

from ......core.datetime_utils import serialize_datetime
from ......core.pydantic_utilities import deep_union_pydantic_dicts
from .rate_entry import RateEntry


class ValidationError_OverlappingRateEntries(pydantic.BaseModel):
    message: str
    rate_a: RateEntry
    rate_b: RateEntry
    type: typing.Literal["overlapping_rate_entries"] = "overlapping_rate_entries"

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.forbid
        json_encoders = {dt.datetime: serialize_datetime}


class ValidationError_VersionConflict(pydantic.BaseModel):
    entity_name: str
    type: typing.Literal["version_conflict"] = "version_conflict"

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.forbid
        json_encoders = {dt.datetime: serialize_datetime}


class ValidationError_OrganizationProviderNotFound(pydantic.BaseModel):
    id: str
    type: typing.Literal["organization_provider_not_found"] = "organization_provider_not_found"

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.forbid
        json_encoders = {dt.datetime: serialize_datetime}


class ValidationError_DuplicateRate(pydantic.BaseModel):
    type: typing.Literal["duplicate_rate"] = "duplicate_rate"

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.forbid
        json_encoders = {dt.datetime: serialize_datetime}


class ValidationError_EmptyEntries(pydantic.BaseModel):
    type: typing.Literal["empty_entries"] = "empty_entries"

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.forbid
        json_encoders = {dt.datetime: serialize_datetime}


ValidationError = typing.Union[
    ValidationError_OverlappingRateEntries,
    ValidationError_VersionConflict,
    ValidationError_OrganizationProviderNotFound,
    ValidationError_DuplicateRate,
    ValidationError_EmptyEntries,
]



# This file was auto-generated by Fern from our API Definition.

from .types import Contact, ExternalProvenance, FilingOrder, MaritalStatus, MutablePatient, Patient, PatientId

__all__ = ["Contact", "ExternalProvenance", "FilingOrder", "MaritalStatus", "MutablePatient", "Patient", "PatientId"]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ....core.datetime_utils import serialize_datetime
from ....core.pydantic_utilities import deep_union_pydantic_dicts
from ...insurance_cards.resources.v_2.types.insurance_card import InsuranceCard
from .individual_id import IndividualId
from .subscriber_base import SubscriberBase


class Subscriber(SubscriberBase):
    """
    Examples
    --------
    import datetime
    import uuid

    from candid import (
        Gender,
        InsuranceTypeCode,
        PatientRelationshipToInsuredCodeAll,
        SourceOfPaymentCode,
        State,
        StreetAddressShortZip,
        Subscriber,
    )
    from candid.resources.insurance_cards.v_2 import InsuranceCard

    Subscriber(
        individual_id=uuid.UUID(
            "797348a9-e7e8-4e59-8628-95390d079c0b",
        ),
        insurance_card=InsuranceCard(
            insurance_card_id=uuid.UUID(
                "ca5b7711-4419-4161-9b7c-3494ac40c8d4",
            ),
            member_id="E85313B4-0FFC-4119-8042-8161A4ECFF0A",
            payer_name="John Doe",
            payer_id="836DDAA6-863F-4020-ACCA-205A689F0002",
            rx_bin="610014",
            rx_pcn="MEDDPRIME",
            image_url_front="https://s3.amazonaws.com/front.jpg",
            image_url_back="https://s3.amazonaws.com/back.jpg",
            group_number="ABC12345",
            plan_name="Silver PPO Plan",
            plan_type=SourceOfPaymentCode.SELF_PAY,
            insurance_type=InsuranceTypeCode.C_12,
        ),
        patient_relationship_to_subscriber_code=PatientRelationshipToInsuredCodeAll.SPOUSE,
        date_of_birth=datetime.date.fromisoformat(
            "2000-01-01",
        ),
        address=StreetAddressShortZip(
            address_1="123 Main St",
            address_2="Apt 1",
            city="New York",
            state=State.NY,
            zip_code="10001",
            zip_plus_four_code="1234",
        ),
        first_name="John",
        last_name="Doe",
        gender=Gender.MALE,
    )
    """

    individual_id: IndividualId
    insurance_card: InsuranceCard

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        populate_by_name = True
        extra = pydantic.Extra.forbid
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .types import (
    Identifier,
    IdentifierBase,
    IdentifierCode,
    IdentifierCreate,
    IdentifierId,
    IdentifierUpdate,
    IdentifierValue,
    IdentifierValue_MedicaidProviderIdentifier,
    IdentifierValue_MedicareProviderIdentifier,
    MedicaidProviderIdentifier,
    MedicareProviderIdentifier,
    UpdatableIdentifier,
    UpdatableIdentifier_Add,
    UpdatableIdentifier_Remove,
    UpdatableIdentifier_Update,
)

__all__ = [
    "Identifier",
    "IdentifierBase",
    "IdentifierCode",
    "IdentifierCreate",
    "IdentifierId",
    "IdentifierUpdate",
    "IdentifierValue",
    "IdentifierValue_MedicaidProviderIdentifier",
    "IdentifierValue_MedicareProviderIdentifier",
    "MedicaidProviderIdentifier",
    "MedicareProviderIdentifier",
    "UpdatableIdentifier",
    "UpdatableIdentifier_Add",
    "UpdatableIdentifier_Remove",
    "UpdatableIdentifier_Update",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ....core.datetime_utils import serialize_datetime
from ....core.pydantic_utilities import deep_union_pydantic_dicts
from ...commons.types.claim_id import ClaimId
from ...era.types.era import Era
from ...service_lines.resources.v_2.types.service_line import ServiceLine
from .claim_status import ClaimStatus


class Claim(pydantic.BaseModel):
    """
    Examples
    --------
    import datetime
    import uuid

    from candid import (
        Claim,
        ClaimStatus,
        DateRangeOptionalEnd,
        Era,
        FacilityTypeCode,
        ProcedureModifier,
        ServiceLineUnits,
    )
    from candid.resources.service_lines.v_2 import (
        DenialReasonContent,
        ServiceLine,
        ServiceLineDenialReason,
    )

    Claim(
        claim_id=uuid.UUID(
            "dd9d7f82-37b5-449d-aa63-26925398335b",
        ),
        status=ClaimStatus.BILLER_RECEIVED,
        clearinghouse="Change Healthcare",
        clearinghouse_claim_id="5BA7C3AB-2BC2-496C-8B10-6CAC73D0729D",
        payer_claim_id="9BB9F259-9756-4F16-8F53-9DBB9F7EB1BB",
        service_lines=[
            ServiceLine(
                modifiers=[ProcedureModifier.TWENTY_TWO],
                charge_amount_cents=10000,
                allowed_amount_cents=8000,
                paid_amount_cents=8000,
                patient_responsibility_cents=2000,
                diagnosis_id_zero=uuid.UUID(
                    "4ac84bcd-12f5-4f86-a57b-e06749127c98",
                ),
                diagnosis_id_one=uuid.UUID(
                    "eea5ca5a-8b43-45fd-8cbd-c6cc1103e759",
                ),
                diagnosis_id_two=uuid.UUID(
                    "5c4aa029-2db9-4202-916e-e93c708f65ff",
                ),
                diagnosis_id_three=uuid.UUID(
                    "81795126-a3ac-443c-b47e-7259a16ab4a2",
                ),
                denial_reason=ServiceLineDenialReason(
                    reason=DenialReasonContent.AUTHORIZATION_REQUIRED,
                ),
                place_of_service_code=FacilityTypeCode.PHARMACY,
                service_line_id=uuid.UUID(
                    "ced00f23-6e68-4678-9dbc-f5aa2969a565",
                ),
                procedure_code="99213",
                quantity="1",
                units=ServiceLineUnits.MJ,
                claim_id=uuid.UUID(
                    "026a1fb8-748e-4859-a2d7-3ea9e07d25ae",
                ),
                date_of_service_range=DateRangeOptionalEnd(
                    start_date="2023-01-01",
                    end_date="2023-01-03",
                ),
                date_of_service=datetime.date.fromisoformat(
                    "2023-01-01",
                ),
                end_date_of_service=datetime.date.fromisoformat(
                    "2023-01-03",
                ),
            )
        ],
        eras=[
            Era(
                era_id=uuid.UUID(
                    "4d844ef1-2253-43cd-a4f1-6db7e65cb54b",
                ),
                check_number="CHK12345",
                check_date="2023-10-12",
            )
        ],
    )
    """

    claim_id: ClaimId
    status: ClaimStatus
    clearinghouse: typing.Optional[str] = None
    clearinghouse_claim_id: typing.Optional[str] = None
    payer_claim_id: typing.Optional[str] = None
    clia_number: typing.Optional[str] = None
    service_lines: typing.List[ServiceLine]
    eras: typing.List[Era]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.forbid
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class BillableStatusType(str, enum.Enum):
    BILLABLE = "BILLABLE"
    NOT_BILLABLE = "NOT_BILLABLE"

    def visit(self, billable: typing.Callable[[], T_Result], not_billable: typing.Callable[[], T_Result]) -> T_Result:
        if self is BillableStatusType.BILLABLE:
            return billable()
        if self is BillableStatusType.NOT_BILLABLE:
            return not_billable()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class TaskSortOptions(str, enum.Enum):
    UPDATED_AT_ASC = "updated_at:asc"
    UPDATED_AT_DESC = "updated_at:desc"
    PATIENT_NAME_ASC = "patient_name:asc"
    PATIENT_NAME_DESC = "patient_name:desc"
    PATIENT_EXTERNAL_ID_ASC = "patient_external_id:asc"
    PATIENT_EXTERNAL_ID_DESC = "patient_external_id:desc"
    PAYER_NAME_ASC = "payer_name:asc"
    PAYER_NAME_DESC = "payer_name:desc"
    PAYER_ID_ASC = "payer_id:asc"
    PAYER_ID_DESC = "payer_id:desc"
    STATUS_ASC = "status:asc"
    STATUS_DESC = "status:desc"
    TASK_TYPE_ASC = "task_type:asc"
    TASK_TYPE_DESC = "task_type:desc"
    CATEGORY_ASC = "category:asc"
    CATEGORY_DESC = "category:desc"
    AGG_UPDATED_AT_ASC = "agg_updated_at:asc"
    AGG_UPDATED_AT_DESC = "agg_updated_at:desc"
    DATE_OF_SERVICE_ASC = "date_of_service:asc"
    DATE_OF_SERVICE_DESC = "date_of_service:desc"
    BLOCKS_CLAIM_SUBMISSION_ASC = "blocks_claim_submission:asc"
    BLOCKS_CLAIM_SUBMISSION_DESC = "blocks_claim_submission:desc"

    def visit(
        self,
        updated_at_asc: typing.Callable[[], T_Result],
        updated_at_desc: typing.Callable[[], T_Result],
        patient_name_asc: typing.Callable[[], T_Result],
        patient_name_desc: typing.Callable[[], T_Result],
        patient_external_id_asc: typing.Callable[[], T_Result],
        patient_external_id_desc: typing.Callable[[], T_Result],
        payer_name_asc: typing.Callable[[], T_Result],
        payer_name_desc: typing.Callable[[], T_Result],
        payer_id_asc: typing.Callable[[], T_Result],
        payer_id_desc: typing.Callable[[], T_Result],
        status_asc: typing.Callable[[], T_Result],
        status_desc: typing.Callable[[], T_Result],
        task_type_asc: typing.Callable[[], T_Result],
        task_type_desc: typing.Callable[[], T_Result],
        category_asc: typing.Callable[[], T_Result],
        category_desc: typing.Callable[[], T_Result],
        agg_updated_at_asc: typing.Callable[[], T_Result],
        agg_updated_at_desc: typing.Callable[[], T_Result],
        date_of_service_asc: typing.Callable[[], T_Result],
        date_of_service_desc: typing.Callable[[], T_Result],
        blocks_claim_submission_asc: typing.Callable[[], T_Result],
        blocks_claim_submission_desc: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is TaskSortOptions.UPDATED_AT_ASC:
            return updated_at_asc()
        if self is TaskSortOptions.UPDATED_AT_DESC:
            return updated_at_desc()
        if self is TaskSortOptions.PATIENT_NAME_ASC:
            return patient_name_asc()
        if self is TaskSortOptions.PATIENT_NAME_DESC:
            return patient_name_desc()
        if self is TaskSortOptions.PATIENT_EXTERNAL_ID_ASC:
            return patient_external_id_asc()
        if self is TaskSortOptions.PATIENT_EXTERNAL_ID_DESC:
            return patient_external_id_desc()
        if self is TaskSortOptions.PAYER_NAME_ASC:
            return payer_name_asc()
        if self is TaskSortOptions.PAYER_NAME_DESC:
            return payer_name_desc()
        if self is TaskSortOptions.PAYER_ID_ASC:
            return payer_id_asc()
        if self is TaskSortOptions.PAYER_ID_DESC:
            return payer_id_desc()
        if self is TaskSortOptions.STATUS_ASC:
            return status_asc()
        if self is TaskSortOptions.STATUS_DESC:
            return status_desc()
        if self is TaskSortOptions.TASK_TYPE_ASC:
            return task_type_asc()
        if self is TaskSortOptions.TASK_TYPE_DESC:
            return task_type_desc()
        if self is TaskSortOptions.CATEGORY_ASC:
            return category_asc()
        if self is TaskSortOptions.CATEGORY_DESC:
            return category_desc()
        if self is TaskSortOptions.AGG_UPDATED_AT_ASC:
            return agg_updated_at_asc()
        if self is TaskSortOptions.AGG_UPDATED_AT_DESC:
            return agg_updated_at_desc()
        if self is TaskSortOptions.DATE_OF_SERVICE_ASC:
            return date_of_service_asc()
        if self is TaskSortOptions.DATE_OF_SERVICE_DESC:
            return date_of_service_desc()
        if self is TaskSortOptions.BLOCKS_CLAIM_SUBMISSION_ASC:
            return blocks_claim_submission_asc()
        if self is TaskSortOptions.BLOCKS_CLAIM_SUBMISSION_DESC:
            return blocks_claim_submission_desc()



# This file was auto-generated by Fern from our API Definition.

from .types import Diagnosis, DiagnosisCreate, DiagnosisId, DiagnosisTypeCode, StandaloneDiagnosisCreate

__all__ = ["Diagnosis", "DiagnosisCreate", "DiagnosisId", "DiagnosisTypeCode", "StandaloneDiagnosisCreate"]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    DenialReasonContent,
    DrugIdentification,
    MeasurementUnitCode,
    ServiceIdQualifier,
    ServiceLine,
    ServiceLineAdjustment,
    ServiceLineCreate,
    ServiceLineCreateStandalone,
    ServiceLineDenialReason,
    ServiceLineUpdate,
)

__all__ = [
    "DenialReasonContent",
    "DrugIdentification",
    "MeasurementUnitCode",
    "ServiceIdQualifier",
    "ServiceLine",
    "ServiceLineAdjustment",
    "ServiceLineCreate",
    "ServiceLineCreateStandalone",
    "ServiceLineDenialReason",
    "ServiceLineUpdate",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ........core.datetime_utils import serialize_datetime
from ........core.pydantic_utilities import deep_union_pydantic_dicts
from .....common.types.address import Address
from .....common.types.contact_point import ContactPoint
from .....common.types.disability_status import DisabilityStatus
from .....common.types.ethnicity import Ethnicity
from .....common.types.external_provider import ExternalProvider
from .....common.types.gender import Gender
from .....common.types.human_name import HumanName
from .....common.types.race import Race
from .....common.types.sex import Sex
from .....common.types.sexual_orientation import SexualOrientation
from .contact import Contact
from .external_provenance import ExternalProvenance
from .filing_order import FilingOrder
from .marital_status import MaritalStatus


class MutablePatient(pydantic.BaseModel):
    """
    An object representing patient demographics information.
    """

    name: HumanName
    other_names: typing.List[HumanName] = pydantic.Field()
    """
    Other names for the patient.
    """

    gender: typing.Optional[Gender] = None
    birth_date: dt.date
    social_security_number: typing.Optional[str] = None
    biological_sex: Sex = pydantic.Field()
    """
    The biological sex of the patient. This corresponds to the HL7 AdministrativeGender https://www.hl7.org/fhir/valueset-administrative-gender.html
    """

    sexual_orientation: typing.Optional[SexualOrientation] = pydantic.Field(default=None)
    """
    The sexual orientation of the patient.
    """

    race: typing.Optional[Race] = None
    ethnicity: typing.Optional[Ethnicity] = None
    disability_status: typing.Optional[DisabilityStatus] = None
    marital_status: typing.Optional[MaritalStatus] = None
    deceased: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Time of death for the patient. Leave unset if the patient is not deceased.
    """

    multiple_birth: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of siblings the patient was born with. Leave unset if the patient was not part of a multiple birth.
    """

    primary_address: Address = pydantic.Field()
    """
    The primary address for the patient.
    """

    other_addresses: typing.List[Address] = pydantic.Field()
    """
    Other addresses for the patient.
    """

    primary_telecom: ContactPoint = pydantic.Field()
    """
    The primary phone number for the patient.
    """

    other_telecoms: typing.List[ContactPoint] = pydantic.Field()
    """
    Other phone numbers for the patient.
    """

    email: typing.Optional[str] = None
    electronic_communication_opt_in: typing.Optional[bool] = None
    photo: typing.Optional[str] = None
    language: typing.Optional[str] = None
    external_provenance: typing.Optional[ExternalProvenance] = pydantic.Field(default=None)
    """
    Information about the upstream system that owns this patient data. Leave unset if Candid owns patient data.
    """

    contacts: typing.List[Contact] = pydantic.Field()
    """
    Contacts for the patient.
    """

    general_practitioners: typing.List[ExternalProvider]
    filing_order: FilingOrder

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.forbid
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ......core.datetime_utils import serialize_datetime
from ......core.pydantic_utilities import deep_union_pydantic_dicts
from .....commons.types.decimal import Decimal
from .....commons.types.facility_type_code import FacilityTypeCode
from .....commons.types.procedure_modifier import ProcedureModifier
from .....commons.types.service_line_units import ServiceLineUnits
from .....encounter_providers.resources.v_2.types.initial_referring_provider import InitialReferringProvider
from .....encounter_providers.resources.v_2.types.ordering_provider import OrderingProvider
from .....encounter_providers.resources.v_2.types.referring_provider import ReferringProvider
from .....encounter_providers.resources.v_2.types.supervising_provider import SupervisingProvider
from .drug_identification import DrugIdentification


class ServiceLineCreate(pydantic.BaseModel):
    modifiers: typing.Optional[typing.List[ProcedureModifier]] = None
    procedure_code: str
    quantity: Decimal = pydantic.Field()
    """
    String representation of a Decimal that can be parsed by most libraries.
    A ServiceLine quantity cannot contain more than one digit of precision.
    Example: 1.1 is valid, 1.11 is not.
    """

    units: ServiceLineUnits
    charge_amount_cents: typing.Optional[int] = pydantic.Field(default=None)
    """
    The total amount charged for this service line taking quantity into account. For example, if a single unit
    costs 100 cents and 2 units were rendered, the charge_amount_cents should be 200. Should be greater than or
    equal to 0.
    """

    diagnosis_pointers: typing.List[int] = pydantic.Field()
    """
    Indices (zero-indexed) of all the diagnoses this service line references
    """

    drug_identification: typing.Optional[DrugIdentification] = None
    place_of_service_code: typing.Optional[FacilityTypeCode] = None
    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    A free-form description to clarify the related data elements and their content. Maps to SV1-01, C003-07 on the 837-P.
    """

    date_of_service: typing.Optional[dt.date] = None
    end_date_of_service: typing.Optional[dt.date] = None
    referring_provider: typing.Optional[ReferringProvider] = pydantic.Field(default=None)
    """
    The final provider who referred the services that were rendered.
    All physicians who order services or refer Medicare beneficiaries must
    report this data.
    """

    initial_referring_provider: typing.Optional[InitialReferringProvider] = pydantic.Field(default=None)
    """
    The second iteration of Loop ID-2310. Use code "P3 - Primary Care Provider" in this loop to
    indicate the initial referral from the primary care provider or whatever provider wrote the initial referral for this patient's episode of care being billed/reported in this transaction.
    Information in Loop ID-2310 applies to the entire claim unless overridden on a service line by the presence of Loop ID-2420 with the same value in NM101.
    """

    supervising_provider: typing.Optional[SupervisingProvider] = pydantic.Field(default=None)
    """
    Required when the rendering provider is supervised by a physician. If not required by this implementation guide, do not send.
    Information in Loop ID-2310 applies to the entire claim unless overridden on a service line by the presence of Loop ID-2420 with the same value in NM101.
    """

    ordering_provider: typing.Optional[OrderingProvider] = pydantic.Field(default=None)
    """
    Required when the service or supply was ordered by a provider who is different than the rendering provider for this service line.
    If not required by this implementation guide, do not send.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.forbid
        json_encoders = {dt.datetime: serialize_datetime}
